@@ -9,7 +9,6 @@ __docformat__ = 'restructuredtext en'
 
 import json, traceback, posixpath, importlib, os
 from io import BytesIO
-from itertools import izip
 
 from calibre import prints
 from calibre.constants import iswindows, numeric_version
@@ -18,16 +17,19 @@ from calibre.devices.mtp.base import debug
 from calibre.devices.mtp.defaults import DeviceDefaults
 from calibre.ptempfile import SpooledTemporaryFile, PersistentTemporaryDirectory
 from calibre.utils.filenames import shorten_components_to
+from polyglot.builtins import iteritems, itervalues, unicode_type, zip
 
 BASE = importlib.import_module('calibre.devices.mtp.%s.driver'%(
     'windows' if iswindows else 'unix')).MTP_DEVICE
 
+
 class MTPInvalidSendPathError(PathError):
 
     def __init__(self, folder):
         PathError.__init__(self, 'Trying to send to ignored folder: %s'%folder)
         self.folder = folder
 
+
 class MTP_DEVICE(BASE):
 
     METADATA_CACHE = 'metadata.calibre'
@@ -62,7 +64,7 @@ class MTP_DEVICE(BASE):
             p.defaults['format_map'] = self.FORMATS
             p.defaults['send_to'] = ['Calibre_Companion', 'Books',
                     'eBooks/import', 'eBooks', 'wordplayer/calibretransfer',
-                    'sdcard/ebooks', 'kindle']
+                    'sdcard/ebooks', 'kindle', 'NOOK']
             p.defaults['send_template'] = '{title} - {authors}'
             p.defaults['blacklist'] = []
             p.defaults['history'] = {}
@@ -73,7 +75,7 @@ class MTP_DEVICE(BASE):
 
     def is_folder_ignored(self, storage_or_storage_id, path,
                           ignored_folders=None):
-        storage_id = unicode(getattr(storage_or_storage_id, 'object_id',
+        storage_id = unicode_type(getattr(storage_or_storage_id, 'object_id',
                              storage_or_storage_id))
         lpath = tuple(icu_lower(name) for name in path)
         if ignored_folders is None:
@@ -164,14 +166,14 @@ class MTP_DEVICE(BASE):
                 traceback.print_exc()
                 dinfo = {}
         if dinfo.get('device_store_uuid', None) is None:
-            dinfo['device_store_uuid'] = unicode(uuid.uuid4())
+            dinfo['device_store_uuid'] = unicode_type(uuid.uuid4())
         if dinfo.get('device_name', None) is None:
             dinfo['device_name'] = self.current_friendly_name
         if name is not None:
             dinfo['device_name'] = name
         dinfo['location_code'] = location_code
         dinfo['last_library_uuid'] = getattr(self, 'current_library_uuid', None)
-        dinfo['calibre_version'] = '.'.join([unicode(i) for i in numeric_version])
+        dinfo['calibre_version'] = '.'.join([unicode_type(i) for i in numeric_version])
         dinfo['date_last_connected'] = isoformat(now())
         dinfo['mtp_prefix'] = storage.storage_prefix
         raw = json.dumps(dinfo, default=to_json)
@@ -227,7 +229,7 @@ class MTP_DEVICE(BASE):
         steps = len(all_books) + 2
         count = 0
 
-        self.report_progress(0, _('Reading ebook metadata'))
+        self.report_progress(0, _('Reading e-book metadata'))
         # Read the cache if it exists
         storage = self.filesystem_cache.storage(sid)
         cache = storage.find_path(self.calibre_file_paths['metadata'].split('/'))
@@ -274,7 +276,7 @@ class MTP_DEVICE(BASE):
             book.path = mtp_file.mtp_id_path
 
         # Remove books in the cache that no longer exist
-        for idx in sorted(relpath_cache.itervalues(), reverse=True):
+        for idx in sorted(itervalues(relpath_cache), reverse=True):
             del bl[idx]
             need_sync = True
 
@@ -341,7 +343,7 @@ class MTP_DEVICE(BASE):
             if iswindows:
                 plen = len(base)
                 name = ''.join(shorten_components_to(245-plen, [name]))
-            with open(os.path.join(base, name), 'wb') as out:
+            with lopen(os.path.join(base, name), 'wb') as out:
                 try:
                     self.get_mtp_file(f, out)
                 except Exception as e:
@@ -418,7 +420,7 @@ class MTP_DEVICE(BASE):
 
         routing = {fmt:dest for fmt,dest in self.get_pref('rules')}
 
-        for infile, fname, mi in izip(files, names, metadata):
+        for infile, fname, mi in zip(files, names, metadata):
             path = self.create_upload_path(prefix, mi, fname, routing)
             if path and self.is_folder_ignored(storage, path):
                 raise MTPInvalidSendPathError('/'.join(path))
@@ -453,7 +455,7 @@ class MTP_DEVICE(BASE):
 
         i, total = 0, len(mtp_files)
         self.report_progress(0, _('Adding books to device metadata listing...'))
-        for x, mi in izip(mtp_files, metadata):
+        for x, mi in zip(mtp_files, metadata):
             mtp_file, bl_idx = x
             bl = booklists[bl_idx]
             book = Book(mtp_file.storage_id, '/'.join(mtp_file.mtp_relpath),
@@ -493,6 +495,7 @@ class MTP_DEVICE(BASE):
 
     def remove_books_from_metadata(self, paths, booklists):
         self.report_progress(0, _('Removing books from metadata'))
+
         class NextPath(Exception):
             pass
 
@@ -531,6 +534,7 @@ class MTP_DEVICE(BASE):
 
     def settings(self):
         class Opts(object):
+
             def __init__(s):
                 s.format_map = self.get_pref('format_map')
         return Opts()
@@ -542,7 +546,7 @@ class MTP_DEVICE(BASE):
     def get_user_blacklisted_devices(self):
         bl = frozenset(self.prefs['blacklist'])
         ans = {}
-        for dev, x in self.prefs['history'].iteritems():
+        for dev, x in iteritems(self.prefs['history']):
             name = x[0]
             if dev in bl:
                 ans[dev] = name
@@ -568,6 +572,6 @@ if __name__ == '__main__':
         dev.set_progress_reporter(prints)
         dev.open(cd, None)
         dev.filesystem_cache.dump()
-        print ('Prefix for main mem:', dev.prefix_for_location(None))
+        print('Prefix for main mem:', dev.prefix_for_location(None))
     finally:
         dev.shutdown()
