@@ -21,6 +21,7 @@ from calibre.gui2.tweak_book.editor.text import TextEdit
 from calibre.utils.icu import sort_key
 from calibre.web.feeds.recipes.collection import get_builtin_recipe_collection, get_builtin_recipe_by_id
 from calibre.utils.localization import localize_user_manual_link
+from polyglot.builtins import iteritems, unicode_type, range
 
 
 def is_basic_recipe(src):
@@ -69,7 +70,7 @@ class CustomRecipeModel(QAbstractListModel):  # {{{
 
     def replace_many_by_title(self, scriptmap):
         script_urn_map = {}
-        for title, script in scriptmap.iteritems():
+        for title, script in iteritems(scriptmap):
             urn = None
             for x in self.recipe_model.custom_recipe_collection:
                 if x.get('title', False) == title:
@@ -118,14 +119,14 @@ def py3_repr(x):
     ans = repr(x)
     if isinstance(x, bytes) and not ans.startswith('b'):
         ans = 'b' + ans
-    if isinstance(x, unicode) and ans.startswith('u'):
+    if isinstance(x, unicode_type) and ans.startswith('u'):
         ans = ans[1:]
     return ans
 
 
 def options_to_recipe_source(title, oldest_article, max_articles_per_feed, feeds):
     classname = 'BasicUserRecipe%d' % int(time.time())
-    title = unicode(title).strip() or classname
+    title = unicode_type(title).strip() or classname
     indent = ' ' * 8
     if feeds:
         if len(feeds[0]) == 1:
@@ -382,7 +383,7 @@ class BasicRecipe(QWidget):  # {{{
 
         def fget(self):
             title = self.title.text().strip()
-            feeds = [self.feeds.item(i).data(Qt.UserRole) for i in xrange(self.feeds.count())]
+            feeds = [self.feeds.item(i).data(Qt.UserRole) for i in range(self.feeds.count())]
             return options_to_recipe_source(title, self.oldest_article.value(), self.max_articles.value(), feeds)
 
         def fset(self, src):
@@ -390,7 +391,7 @@ class BasicRecipe(QWidget):  # {{{
             self.feed_title.clear()
             self.feed_url.clear()
             if src is None:
-                self.title.setText(_('My News Source'))
+                self.title.setText(_('My news source'))
                 self.oldest_article.setValue(7)
                 self.max_articles.setValue(100)
             else:
@@ -494,7 +495,7 @@ class CustomRecipes(Dialog):
         else:
             bb.setStandardButtons(bb.Cancel | bb.Save)
             if self.stack.currentIndex() == 1:
-                text = _('S&witch to Advanced mode')
+                text = _('S&witch to advanced mode')
                 tooltip = _('Edit this recipe in advanced mode')
                 receiver = self.switch_to_advanced
                 b = bb.addButton(text, bb.ActionRole)
@@ -561,7 +562,7 @@ class CustomRecipes(Dialog):
         d.l = QVBoxLayout()
         d.setLayout(d.l)
         d.list = QListWidget(d)
-        d.list.doubleClicked.connect(lambda x: d.accept())
+        connect_lambda(d.list.doubleClicked, d, lambda d: d.accept())
         d.l.addWidget(d.list)
         d.bb = QDialogButtonBox(QDialogButtonBox.Ok|QDialogButtonBox.Cancel,
                 Qt.Horizontal, d)
@@ -593,8 +594,8 @@ class CustomRecipes(Dialog):
         if not items:
             return
         item = items[-1]
-        id_ = unicode(item.data(Qt.UserRole) or '')
-        title = unicode(item.data(Qt.DisplayRole) or '').rpartition(' [')[0]
+        id_ = unicode_type(item.data(Qt.UserRole) or '')
+        title = unicode_type(item.data(Qt.DisplayRole) or '').rpartition(' [')[0]
         src = get_builtin_recipe_by_id(id_, download_recipe=True)
         if src is None:
             raise Exception('Something weird happened')
@@ -649,7 +650,7 @@ class CustomRecipes(Dialog):
         if replace_recipes:
             self.recipe_list.replace_many_by_title(replace_recipes)
         if failed_recipes:
-            det_msg = '\n'.join('%s\n%s\n' % (title, tb) for title, tb in failed_recipes.iteritems())
+            det_msg = '\n'.join('%s\n%s\n' % (title, tb) for title, tb in iteritems(failed_recipes))
             error_dialog(self, _('Failed to create recipes'), _(
                 'Failed to create some recipes, click "Show details" for details'), show=True,
                          det_msg=det_msg)
