@@ -16,6 +16,8 @@ from PyQt5.Qt import (
 )
 
 from calibre.gui2 import error_dialog
+from polyglot.builtins import iteritems, unicode_type
+
 
 class CreateCustomColumn(QDialog):
 
@@ -25,17 +27,17 @@ class CreateCustomColumn(QDialog):
     column_types = dict(enumerate((
         {
             'datatype':'text',
-            'text':_('Text, column shown in the tag browser'),
+            'text':_('Text, column shown in the Tag browser'),
             'is_multiple':False
         },
         {
             'datatype':'*text',
-            'text':_('Comma separated text, like tags, shown in the tag browser'),
+            'text':_('Comma separated text, like tags, shown in the Tag browser'),
             'is_multiple':True
         },
         {
             'datatype':'comments',
-            'text':_('Long text, like comments, not shown in the tag browser'),
+            'text':_('Long text, like comments, not shown in the Tag browser'),
             'is_multiple':False
         },
         {
@@ -84,7 +86,7 @@ class CreateCustomColumn(QDialog):
             'is_multiple':True
         },
     )))
-    column_types_map = {k['datatype']:idx for idx, k in column_types.iteritems()}
+    column_types_map = {k['datatype']:idx for idx, k in iteritems(column_types)}
 
     def __init__(self, parent, current_row, current_key, standard_colheads, standard_colnames):
         QDialog.__init__(self, parent)
@@ -111,7 +113,7 @@ class CreateCustomColumn(QDialog):
             self.column_type_box.addItem(self.column_types[t]['text'])
         self.column_type_box.currentIndexChanged.connect(self.datatype_changed)
 
-        all_colors = [unicode(s) for s in list(QColor.colorNames())]
+        all_colors = [unicode_type(s) for s in list(QColor.colorNames())]
         self.enum_colors_label.setToolTip('<p>' + ', '.join(all_colors) + '</p>')
 
         if not self.editing_col:
@@ -119,8 +121,8 @@ class CreateCustomColumn(QDialog):
             self.exec_()
             return
 
-        self.setWindowTitle(_('Edit a custom column'))
-        self.heading_label.setText('<b>' + _('Edit a custom column'))
+        self.setWindowTitle(_('Edit custom column'))
+        self.heading_label.setText('<b>' + _('Edit custom column'))
         self.shortcuts.setVisible(False)
         idx = current_row
         if idx < 0:
@@ -135,6 +137,7 @@ class CreateCustomColumn(QDialog):
         c = parent.custcols[col]
         self.column_name_box.setText(c['label'])
         self.column_heading_box.setText(c['name'])
+        self.column_heading_box.setFocus()
         ct = c['datatype']
         if c['is_multiple']:
             ct = '*' + ct
@@ -183,7 +186,7 @@ class CreateCustomColumn(QDialog):
         self.exec_()
 
     def shortcut_activated(self, url):  # {{{
-        which = unicode(url).split(':')[-1]
+        which = unicode_type(url).split(':')[-1]
         self.column_type_box.setCurrentIndex({
             'yesno': self.column_types_map['bool'],
             'tags' : self.column_types_map['*text'],
@@ -208,7 +211,7 @@ class CreateCustomColumn(QDialog):
             self.composite_box.setText(
                 {
                     'isbn': '{identifiers:select(isbn)}',
-                    'formats': "{:'approximate_formats()'}",
+                    'formats': "{:'re(approximate_formats(), ',', ', ')'}",
                     }[which])
             self.composite_sort_by.setCurrentIndex(0)
         if which == 'text':
@@ -270,7 +273,7 @@ class CreateCustomColumn(QDialog):
 
         # Heading
         self.column_heading_box = chb = QLineEdit(self)
-        chb.setToolTip(_("Column heading in the library view and category name in the tag browser"))
+        chb.setToolTip(_("Column heading in the library view and category name in the Tag browser"))
         add_row(_("Column &heading"), chb)
 
         # Column Type
@@ -299,7 +302,7 @@ class CreateCustomColumn(QDialog):
         self.format_box = fb = QLineEdit(self)
         h.addWidget(fb)
         self.format_default_label = la = QLabel('')
-        la.setOpenExternalLinks(True)
+        la.setOpenExternalLinks(True), la.setWordWrap(True)
         h.addWidget(la)
         self.format_label = add_row('', h)
 
@@ -321,7 +324,7 @@ class CreateCustomColumn(QDialog):
         ):
             ct.addItem(text, k)
         ct.setToolTip(_('Choose whether or not the column heading is shown in the Book\n'
-                        'Details panel and, if shown, where'))
+                        'details panel and, if shown, where'))
         self.comments_heading_position_label = add_row(_('Column heading'), ct)
 
         self.comments_type = ct = QComboBox(self)
@@ -333,7 +336,7 @@ class CreateCustomColumn(QDialog):
         ):
             ct.addItem(text, k)
         ct.setToolTip(_('Choose how the data in this column is interpreted.\n'
-                        'This controls how the data is displayed in the Book Details panel\n'
+                        'This controls how the data is displayed in the Book details panel\n'
                         'and how it is edited.'))
         self.comments_type_label = add_row(_('Interpret this column as:') + ' ', ct)
 
@@ -366,23 +369,23 @@ class CreateCustomColumn(QDialog):
         self.composite_sort_by = csb = QComboBox(self)
         la.setBuddy(csb), csb.setToolTip(_("How this column should handled in the GUI when sorting and searching"))
         l.addWidget(la), l.addWidget(csb)
-        self.composite_make_category = cmc = QCheckBox(_("Show in tags browser"))
-        cmc.setToolTip(_("If checked, this column will appear in the tags browser as a category"))
+        self.composite_make_category = cmc = QCheckBox(_("Show in Tag browser"))
+        cmc.setToolTip(_("If checked, this column will appear in the Tag browser as a category"))
         l.addWidget(cmc)
-        self.composite_contains_html = cch = QCheckBox(_("Show as HTML in book details"))
-        cch.setToolTip('<p>' +
-                _('If checked, this column will be displayed as HTML in '
-                  'book details and the content server. This can be used to '
-                  'construct links with the template language. For example, '
-                  'the template '
-                  '<pre>&lt;big&gt;&lt;b&gt;{title}&lt;/b&gt;&lt;/big&gt;'
-                  '{series:| [|}{series_index:| [|]]}</pre>'
-                  'will create a field displaying the title in bold large '
-                  'characters, along with the series, for example <br>"<big><b>'
-                  'An Oblique Approach</b></big> [Belisarius [1]]". The template '
-                  '<pre>&lt;a href="http://www.beam-ebooks.de/ebook/{identifiers'
-                  ':select(beam)}"&gt;Beam book&lt;/a&gt;</pre> '
-                  'will generate a link to the book on the Beam ebooks site.') + '</p>')
+        self.composite_contains_html = cch = QCheckBox(_("Show as HTML in Book details"))
+        cch.setToolTip('<p>' + _(
+            'If checked, this column will be displayed as HTML in '
+            'Book details and the Content server. This can be used to '
+            'construct links with the template language. For example, '
+            'the template '
+            '<pre>&lt;big&gt;&lt;b&gt;{title}&lt;/b&gt;&lt;/big&gt;'
+            '{series:| [|}{series_index:| [|]]}</pre>'
+            'will create a field displaying the title in bold large '
+            'characters, along with the series, for example <br>"<big><b>'
+            'An Oblique Approach</b></big> [Belisarius [1]]". The template '
+            '<pre>&lt;a href="https://www.beam-ebooks.de/ebook/{identifiers'
+            ':select(beam)}"&gt;Beam book&lt;/a&gt;</pre> '
+            'will generate a link to the book on the Beam e-books site.') + '</p>')
         l.addWidget(cch)
         add_row(None, l)
 
@@ -401,24 +404,46 @@ class CreateCustomColumn(QDialog):
             if col_type == 'datetime':
                 l, dl = _('&Format for dates'), _('Default: dd MMM yyyy.')
                 self.format_box.setToolTip(_(
-                    "<p>Date format. Use 1-4 \'d\'s for day, 1-4 \'M\'s for month, and 2 or 4 \'y\'s for year.</p>\n"
+                    '<p>Date format.</p>'
+                    '<p>The formatting codes are:'
+                    '<ul>'
+                    '<li>d    : the day as number without a leading zero (1 to 31)</li>'
+                    '<li>dd   : the day as number with a leading zero (01 to 31)</li>'
+                    '<li>ddd  : the abbreviated localized day name (e.g. "Mon" to "Sun").</li>'
+                    '<li>dddd : the long localized day name (e.g. "Monday" to "Sunday").</li>'
+                    '<li>M    : the <b>month</b> as number without a leading zero (1 to 12).</li>'
+                    '<li>MM   : the <b>month</b> as number with a leading zero (01 to 12)</li>'
+                    '<li>MMM  : the abbreviated localized <b>month</b> name (e.g. "Jan" to "Dec").</li>'
+                    '<li>MMMM : the long localized <b>month</b> name (e.g. "January" to "December").</li>'
+                    '<li>yy   : the year as two digit number (00 to 99).</li>'
+                    '<li>yyyy : the year as four digit number.</li>'
+                    '<li>h    : the hours without a leading 0 (0 to 11 or 0 to 23, depending on am/pm)</li>'
+                    '<li>hh   : the hours with a leading 0 (00 to 11 or 00 to 23, depending on am/pm)</li>'
+                    '<li>m    : the <b>minutes</b> without a leading 0 (0 to 59)</li>'
+                    '<li>mm   : the <b>minutes</b> with a leading 0 (00 to 59)</li>'
+                    '<li>s    : the seconds without a leading 0 (0 to 59)</li>'
+                    '<li>ss   : the seconds with a leading 0 (00 to 59)</li>'
+                    '<li>ap   : use a 12-hour clock instead of a 24-hour clock, with "ap" replaced by the localized string for am or pm</li>'
+                    '<li>AP   : use a 12-hour clock instead of a 24-hour clock, with "AP" replaced by the localized string for AM or PM</li>'
+                    '<li>iso  : the date with time and timezone. Must be the only format present</li>'
+                    '</ul></p>'
                     "<p>For example:\n"
                     "<ul>\n"
-                    "<li> ddd, d MMM yyyy gives Mon, 5 Jan 2010<li>\n"
+                    "<li>ddd, d MMM yyyy gives Mon, 5 Jan 2010<li>\n"
                     "<li>dd MMMM yy gives 05 January 10</li>\n"
                     "</ul> "))
             else:
                 l, dl = _('&Format for numbers'), (
                     '<p>' + _('Default: Not formatted. For format language details see'
-                    ' <a href="https://docs.python.org/library/string.html#format-string-syntax">the python documentation</a>'))
+                    ' <a href="https://docs.python.org/library/string.html#format-string-syntax">the Python documentation</a>'))
                 if col_type == 'int':
-                    self.format_box.setToolTip('<p>' +
-                        _('Examples: The format <code>{0:0>4d}</code> '
+                    self.format_box.setToolTip('<p>' + _(
+                        'Examples: The format <code>{0:0>4d}</code> '
                         'gives a 4-digit number with leading zeros. The format '
                         '<code>{0:d}&nbsp;days</code> prints the number then the word "days"')+ '</p>')
                 else:
-                    self.format_box.setToolTip('<p>' +
-                        _('Examples: The format <code>{0:.1f}</code> gives a floating '
+                    self.format_box.setToolTip('<p>' + _(
+                        'Examples: The format <code>{0:.1f}</code> gives a floating '
                         'point number with 1 digit after the decimal point. The format '
                         '<code>Price:&nbsp;$&nbsp;{0:,.2f}</code> prints '
                         '"Price&nbsp;$&nbsp;" then displays the number with 2 digits '
@@ -440,18 +465,18 @@ class CreateCustomColumn(QDialog):
         self.allow_half_stars.setVisible(col_type == 'rating')
 
     def accept(self):
-        col = unicode(self.column_name_box.text()).strip()
+        col = unicode_type(self.column_name_box.text()).strip()
         if not col:
             return self.simple_error('', _('No lookup name was provided'))
         if col.startswith('#'):
             col = col[1:]
-        if re.match('^\w*$', col) is None or not col[0].isalpha() or col.lower() != col:
+        if re.match(r'^\w*$', col) is None or not col[0].isalpha() or col.lower() != col:
             return self.simple_error('', _('The lookup name must contain only '
                     'lower case letters, digits and underscores, and start with a letter'))
         if col.endswith('_index'):
             return self.simple_error('', _('Lookup names cannot end with _index, '
                     'because these names are reserved for the index of a series column.'))
-        col_heading = unicode(self.column_heading_box.text()).strip()
+        col_heading = unicode_type(self.column_heading_box.text()).strip()
         coldef = self.column_types[self.column_type_box.currentIndex()]
         col_type = coldef['datatype']
         if col_type[0] == '*':
@@ -487,33 +512,33 @@ class CreateCustomColumn(QDialog):
         display_dict = {}
 
         if col_type == 'datetime':
-            if unicode(self.format_box.text()).strip():
-                display_dict = {'date_format':unicode(self.format_box.text()).strip()}
+            if unicode_type(self.format_box.text()).strip():
+                display_dict = {'date_format':unicode_type(self.format_box.text()).strip()}
             else:
                 display_dict = {'date_format': None}
         elif col_type == 'composite':
-            if not unicode(self.composite_box.text()).strip():
+            if not unicode_type(self.composite_box.text()).strip():
                 return self.simple_error('', _('You must enter a template for'
                     ' composite columns'))
-            display_dict = {'composite_template':unicode(self.composite_box.text()).strip(),
+            display_dict = {'composite_template':unicode_type(self.composite_box.text()).strip(),
                             'composite_sort': ['text', 'number', 'date', 'bool']
                                         [self.composite_sort_by.currentIndex()],
                             'make_category': self.composite_make_category.isChecked(),
                             'contains_html': self.composite_contains_html.isChecked(),
                         }
         elif col_type == 'enumeration':
-            if not unicode(self.enum_box.text()).strip():
+            if not unicode_type(self.enum_box.text()).strip():
                 return self.simple_error('', _('You must enter at least one'
                     ' value for enumeration columns'))
-            l = [v.strip() for v in unicode(self.enum_box.text()).split(',') if v.strip()]
+            l = [v.strip() for v in unicode_type(self.enum_box.text()).split(',') if v.strip()]
             l_lower = [v.lower() for v in l]
             for i,v in enumerate(l_lower):
                 if v in l_lower[i+1:]:
                     return self.simple_error('', _('The value "{0}" is in the '
                     'list more than once, perhaps with different case').format(l[i]))
-            c = unicode(self.enum_colors.text())
+            c = unicode_type(self.enum_colors.text())
             if c:
-                c = [v.strip() for v in unicode(self.enum_colors.text()).split(',')]
+                c = [v.strip() for v in unicode_type(self.enum_colors.text()).split(',')]
             else:
                 c = []
             if len(c) != 0 and len(c) != len(l):
@@ -528,13 +553,13 @@ class CreateCustomColumn(QDialog):
         elif col_type == 'text' and is_multiple:
             display_dict = {'is_names': self.is_names.isChecked()}
         elif col_type in ['int', 'float']:
-            if unicode(self.format_box.text()).strip():
-                display_dict = {'number_format':unicode(self.format_box.text()).strip()}
+            if unicode_type(self.format_box.text()).strip():
+                display_dict = {'number_format':unicode_type(self.format_box.text()).strip()}
             else:
                 display_dict = {'number_format': None}
         elif col_type == 'comments':
-            display_dict['heading_position'] = type(u'')(self.comments_heading_position.currentData())
-            display_dict['interpret_as'] = type(u'')(self.comments_type.currentData())
+            display_dict['heading_position'] = unicode_type(self.comments_heading_position.currentData())
+            display_dict['interpret_as'] = unicode_type(self.comments_type.currentData())
         elif col_type == 'rating':
             display_dict['allow_half_stars'] = bool(self.allow_half_stars.isChecked())
 
